<?php

/**
 * @file
 *  This is an universalsubtitles provider include file for Embedded Media Video.
 *
 */

/**
 *  This is the main URL for your provider.
 */
define('EMVIDEO_UNIVERSALSUBTITLES_MAIN_URL', 'http://universalsubtitles.org/');

/**
 *  This is some extra documentation URL for your provider.
 */
define('EMVIDEO_UNIVERSALSUBTITLES_EMBED_DOC_URL', 'https://github.com/8planes/mirosubs/wiki/Embed-Code-Usage-Guide');

/**
 *  This is the API documentation URL for your provider.
 */
define('EMVIDEO_UNIVERSALSUBTITLES_API_DOC_URL', 'http://universalsubtitles.org/api/1.0/documentation/');

/**
 *  This is the URL to the API of your provider, if this exists.
 */
define('EMVIDEO_UNIVERSALSUBTITLES_API_URL', 'http://universalsubtitles.org/api/1.0');

/**
 *  The Development API URL
 */
define('EMVIDEO_UNIVERSALSUBTITLES_API_URL_DEV', 'http://dev.universalsubtitles.org/api/1.0');

/**
 *  This defines the version of the content data array that we serialize
 *  in emvideo_universalsubtitles_data(). If we change the expected keys of that array,
 *  we must increment this value, which will allow older content to be updated
 *  to the new version automatically.
 *
 *  @see emvideo_universalsubtitles_data
 */
define('EMVIDEO_UNIVERSALSUBTITLES_DATA_VERSION', 1);

/**
 * hook emvideo_PROVIDER_info
 * This returns information relevant to a specific 3rd party video provider.
 *
 * @return
 *   A keyed array of strings requested by various admin and other forms.
 *    'provider' => The machine name of the provider. This must be the same as
 *      the base name of this filename, before the .inc extension.
 *    'name' => The translated name of the provider.
 *    'url' => The url to the main page for the provider.
 *    'settings_description' => A description of the provider that will be
 *      posted in the admin settings form.
 *    'supported_features' => An array of rows describing the state of certain
 *      supported features by the provider. These will be rendered in a table,
 *      with the columns being 'Feature', 'Supported', 'Notes'. In general,
 *      the 'Feature' column will give the name of the feature, 'Supported'
 *      will be Yes or No, and 'Notes' will give an optional description or
 *      caveats to the feature.
 */
function emvideo_universalsubtitles_info() {
  $features = array(
    array(t('Autoplay'), t('Yes'), ''),
    array(t('RSS Attachment'), t('No'), ''),
    array(t('Thumbnails'), t('Yes'), ''),
    array(t('Full screen mode'), t('No'), t('Universal Subtitles does not yet support fullscreen playback with subtitles.')),
  );
  return array(
    'module' => 'media_universalsubtitles',
    'provides' => array('emvideo'),
    'provider' => 'universalsubtitles',
    'name' => t('Universal Subtitles'),
    'url' => EMVIDEO_UNIVERSALSUBTITLES_MAIN_URL,
    'settings_description' => t('These settings specifically affect videos displayed from !universalsubtitles. You can also read more about its !api and the !embed.',
       array(
         '!universalsubtitles' => l(t('Universal Subtitles'), EMVIDEO_UNIVERSALSUBTITLES_MAIN_URL),
         '!api' => l(t("developer's API"), EMVIDEO_UNIVERSALSUBTITLES_API_DOC_URL),
         '!embed' => l(t("embed examples"), EMVIDEO_UNIVERSALSUBTITLES_EMBED_DOC_URL),
       )
     ),
    'supported_features' => $features,
  );
}

/**
 * Implements hook emvideo_PROVIDER_settings
 */
function emvideo_universalsubtitles_settings() {
  $form['universalsubtitles']['login'] = array(
    '#type' => 'fieldset',
    '#title' => t('Universal Subtitles account settings'),
    '#description' => t('These are global settings'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );
  $form['universalsubtitles']['login']['emvideo_universalsubtitles_use_global_login'] = array(
    '#type' => 'checkbox',
    '#title' => t('Use global login'),
    '#default_value' => variable_get('emvideo_universalsubtitles_use_global_login', 0),
    '#description' => t('Use this when being sure this is ok.'),
  );
  $form['universalsubtitles']['login']['global'] = array(
    '#type' => 'fieldset',
    '#title' => t('Global settings'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );
  $form['universalsubtitles']['login']['global']['emvideo_universalsubtitles_global_username'] = array(
    '#type' => 'textfield',
    '#title' => t('Username'),
    '#default_value' => variable_get('emvideo_universalsubtitles_global_username', ''),
    '#description' => t('Your Universal Subtitles account username.'),
  );
  $form['universalsubtitles']['login']['global']['emvideo_universalsubtitles_global_password'] = array(
    '#type' => 'textfield',
    '#title' => t('Password'),
    '#default_value' => variable_get('emvideo_universalsubtitles_global_password', ''),
    '#description' => t('Your Universal Subtitles account password.'),
  );
  $form['universalsubtitles']['dev_server'] = array(
    '#type' => 'fieldset',
    '#title' => t('Universal Subtitles API settings'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );
  $form['universalsubtitles']['dev_server']['emvideo_universalsubtitles_use_dev_server'] = array(
    '#type' => 'checkbox',
    '#title' => t('Use dev server'),
    '#default_value' => variable_get('emvideo_universalsubtitles_use_dev_server', 0),
    '#description' => t('Use the development server to test the latest features.'),
  );
  $form['universalsubtitles']['player_options'] = array(
    '#type' => 'fieldset',
    '#title' => t('Embedded video player options'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );
  // This is an option to set the video to full screen. You should remove this
  // option if it is not provided by the universalsubtitles provider.
  //$form['universalsubtitles']['player_options']['emvideo_universalsubtitles_full_screen'] = array(
  //  '#type' => 'checkbox',
  //  '#title' => t('Allow fullscreen'),
  //  '#default_value' => variable_get('emvideo_universalsubtitles_full_screen', 1),
  //  '#description' => t('Allow users to view video using the entire computer screen.'),
  //);
  $form['universalsubtitles']['player_options']['emvideo_universalsubtitles_proxy'] = array(
    '#type' => 'checkbox',
    '#title' => t('Proxy for other providers'),
    '#default_value' => variable_get('emvideo_universalsubtitles_proxy', 0),
    '#description' => t('Allow users to link to videos directly from ie youtube. That is UniSub supported providers.<br/>Note: You need to disable other video providers for this field to make this work.'),
  );
  return $form;
}

/**
 *  hook emvideo_PROVIDER_extract
 *
 *  This is called to extract the video code from a pasted URL or embed code.
 *
 *  We'll be passed a URL or the embed code from a video when an editor pastes
 *  that in the field's textfield. We'll need to either pass back an array of
 *  regex expressions to match, or do the matching ourselves and return the
 *  resulting video code.
 *
 *  @see emfield_parse_embed
 *
 *  @param $parse
 *    An optional string with the pasted URL or embed code.
 *
 *  @return
 *    Either an array of regex expressions to be tested, or a string with the
 *    video code to be used. If the hook tests the code itself, it should
 *    return either the string of the video code (if matched), or an empty
 *    array. Otherwise, the calling function will handle testing the embed code
 *    against each regex string in the returned array.
 */
function emvideo_universalsubtitles_extract($parse = '') {
  static $parsed;

  if (empty($parse)) return;

  if (!isset($parsed)) {
    $parsed = array();
  }

  if (isset($parsed[$parse])) {
    return $parsed[$parse];
  }

  $result = NULL;

  // Permalink urls are : http://universalsubtitles.org/nl/videos/NOfZXGeyeP1G
  /*
   * TODO: get API support for premalink
   * We currently do not support for permalinks ... there is no way / API to render the video
   */

  /*
   * For the unisub embed code we need to parse for the video_url
   * then call the API to get to the permalink ID
   */
  $data = _universalsubtitles_api_video($parse);
  if (isset($data['video_id'])) {
    // The permalink ID
    $result = $data['video_id'];
  }

  $parsed[$parse] = $result;

  // No matches
  return $result;
}

function _universalsubtitles_parse_url($url) {
  // TODO: safeguard input URLs
  drupal_set_message(t('We accept the URL as a proxy for %url', array('%url' => $url)), 'notice');
  return $url;
}

/**
 * Processes the UniSub embed code
 *
 *   <script type="text/javascript" src="http://s3.www.universalsubtitles.org/embed.js">
 *   (
 *    {"base_state": {}, "video_url": "http://vimeo.com/18352872"}
 *   )
 *   </script>
 *
 * @param $embed
 *
 * @return
 *   array with key value pairs
 */
function _universalsubtitles_parse_embed($embed) {
  // Check for valid http(s) : http://phpcentral.com/208-url-validation-in-php.html
  $regex = '@<script type="text/javascript" src="http://s3.www.universalsubtitles.org/embed.js">.*video_url"\W*:\W*"(http(s)?://[a-z0-9-]+(.[a-z0-9-]+)*(:[0-9]+)?(/.*)?)".*@i';

  $matches = NULL;
  if (preg_match($regex, trim($embed), $matches)) {
    $video_url = $matches[1];
    $result = array();
    $result['video_url'] = $video_url;
    return $result;
  }
}

/**
 * Grab and transform UniSub struct based on original video URL into emfield data struct
 *
 * The video_url is only available in the embed code from UniSub
 *
 * @param $video_url
 *   This is the URL to the original video ie http://www.youtube.com/watch?v=ZWvKQ9UkGfc
 *
 * @return array
 *   An data struct
 */
function _universalsubtitles_api_video($input) {
  static $files;

  if (!isset($files)) $files = array();

  $values = _universalsubtitles_parse_embed($input);
  if (isset($values) && isset($values['video_url'])) {
    $video_url = $values['video_url'];
  }
  else {
    // We now have either a UniSub permalink
    // or one of the other video providers URL
    // http://www.universalsubtitles.org/en/videos/hecAg8gg5esT/
    $regex = '@http://www.universalsubtitles.org/.*videos/([0-9a-z]+)/@i';
    $matches = NULL;
    if (preg_match($regex, $input, $matches)) {
      drupal_set_message("Use the embed code instead.<br />Permalinks are not completely implemented by the UniSub API yet (they lack the video_url until 2011-04-15).<br />See https://www.pivotaltracker.com/story/show/11233199", 'warning', FALSE);
      $video_id = $matches[1];
    }
    else {
      // Assume $input is a URL from others then UniSub
      if (variable_get('emvideo_universalsubtitles_proxy', FALSE)) {
        $video_url = _universalsubtitles_parse_url($input);
      }
    }
  }

  // empty result
  $data = array();

  $api_url = EMVIDEO_UNIVERSALSUBTITLES_API_URL;
  if (variable_get('emvideo_universalsubtitles_use_dev_server', FALSE)) {
    $api_url = EMVIDEO_UNIVERSALSUBTITLES_API_URL_DEV;
  }

  if (!empty($video_url)) {
    $file = $api_url . '/video/?video_url=' . $video_url;
  }
  else if (!empty($video_id)) {
    // API on http://www.universalsubtitles.org/api/1.0/documentation/
    // - http://www.universalsubtitles.org/api/1.0/video/JVoMAa3kaWzq/
    //TODO: when API implements this by returning video_url
    $file = $api_url . '/video/' . $video_id .'/';
    //return $data;
  }
  else {
    return $data;
  }

  // Cached?
  if (isset($files[$file])) return $files[$file];

  // Grab the json settings
  $use_global_login = variable_get('emvideo_universalsubtitles_use_global_login', FALSE);
  if ($use_global_login) {
    $username = variable_get('emvideo_universalsubtitles_global_username', '');
    $password = variable_get('emvideo_universalsubtitles_global_password', '');
  }
  else {
    // TODO: add user variables for these settings
    $username = '';
    $password = '';
  }
  if ($username && $password) {
    $file_uri = $file . '&username=' . $username . '&password=' . $password;
  }
  else {
    // This URI will always fail ... but we stick with it
    // in case a dev server allows for it
    $file_uri = $file;
  }

  // Make sure to suppress errors as these show username and password
  $content = @file_get_contents($file_uri);
  $unisub = json_decode($content);
  if (is_object($unisub)) {
    /*
     * Current data
     *
     * description (String, 0 characters )
     * created (String, 19 characters ) 2010-11-11 11:04:30
     * video_id (String, 12 characters ) yKsUAgChJlGA
     * title (String, 17 characters ) Switch to Firefox
     * allow_community_edits (Boolean) TRUE
     * homepage (String, 81 characters ) http://www.universalsubtitles.org/videos/yKsUAg...
     * thumbnail (String, 0 characters )
     * allow_video_urls_edit (Boolean) TRUE
     * resource_uri (String, 28 characters ) /api/1.0/video/yKsUAgChJlGA/
     */
    $data['video_id'] = $unisub->video_id;
    if (isset($unisub->video_url)) {
      $video_url = $unisub->video_url;
    }
    $data['video_url'] = $video_url;

    // @see : http://universalsubtitles.org/api/1.0/video/?video_url=http://www.youtube.com/watch?v=kgicuytCkoY
    if (!empty($unisub->thumbnail)) {
      $data['thumbnail']['url'] = $unisub->thumbnail;
      $data['thumbnail']['title'] = $unisub->title;
    }
    // TODO: this is not working: we have no access to the widget object
    if (!empty($unisub->title)) {
      $data['title'] = $unisub->title;
    }
    // TODO: this is not working
    if (!empty($unisub->description)) {
      $data['description'] = $unisub->description;
    }
  }

  $files[$file] = $data;
  return $data;
}

/**
 *  hook emvideo_PROVIDER_data
 *
 *  Provides an array to be serialised and made available with $item elsewhere.
 *
 *  This data can be used to store any extraneous information available
 *  specifically to the universalsubtitles provider.
 *
 *  @see theme_emvideo_video_thumbnail
 */
function emvideo_universalsubtitles_data($field, $item) {
  // Initialize the data array.
  $data = array();

  // Create some version control. Thus if we make changes to the data array
  // down the road, we can respect older content. If allowed by Embedded Media
  // Field, any older content will automatically update this array as needed.
  // In any case, you should account for the version if you increment it.
  $data['emvideo_universalsubtitles_version'] = EMVIDEO_UNIVERSALSUBTITLES_DATA_VERSION;

  // We ask UniSub for the data belonging to the video URL
  $extra_data = _universalsubtitles_api_video($item['embed']);
  if (!empty($extra_data)) {
    $data = array_merge($data, $extra_data);
  }
  else {
    drupal_set_message('Unable to grab metadata for this video from Universal Subtitles');
  }
  if (!isset($data['thumbnail'])) {
    $data['thumbnail']['url'] = "http://s3.www.universalsubtitles.org/images/logo.png";
  }
  return $data;
}

/**
 *  hook emvideo_PROVIDER_rss
 *
 *  This attaches a file to an RSS feed.
 */
function emvideo_universalsubtitles_rss($item, $teaser = NULL) {
  if ($item['value']) {
    $file['thumbnail']['filepath'] = $item['data']['thumbnail'];
    return $file;
  }
}

/**
 * hook emvideo_PROVIDER_embedded_link($video_code)
 * returns a link to view the video at the provider's site.
 *  @param $video_code
 *    The string containing the video to watch.
 *  @return
 *    A string containing the URL to view the video at the original provider's site.
 */
function emvideo_universalsubtitles_embedded_link($video_code) {
  return 'http://universalsubtitles.org/videos/'. $video_code;
}

/**
 * The embedded flash displaying the universalsubtitles video.
 */
function theme_emvideo_universalsubtitles_flash($item, $width, $height, $autoplay) {
  $output = '';
  if ($item['embed']) {
    $autoplay = $autoplay ? 'true' : 'false';
    $fullscreen = variable_get('emvideo_universalsubtitles_full_screen', 1) ? 'true' : 'false';
    $autoplay = $autoplay ? 'true' : 'false';

    $output = '<script type="text/javascript" src="http://s3.www.universalsubtitles.org/embed.js">';
    $output .= '(';
    $output .= '  { "base_state": ';
    $output .= '    {';
    $output .= '      "start_playing": ' . $autoplay;
    $output .= '    },';
    $output .= '    "video_url": "'. $item['data']['video_url'] . '"';
    $output .= '  }';
    $output .= ')';
    $output .= '</script>';
  }
  return $output;
}

/**
 * hook emvideo_PROVIDER_thumbnail
 * Returns the external url for a thumbnail of a specific video.
 *  @param $field
 *    The field of the requesting node.
 *  @param $item
 *    The actual content of the field from the requesting node.
 *  @return
 *    A URL pointing to the thumbnail.
 */
function emvideo_universalsubtitles_thumbnail($field, $item, $formatter, $node, $width, $height) {
  return $item['data']['thumbnail']['url'];
}

/**
 *  hook emvideo_PROVIDER_video
 *  This actually displays the full/normal-sized video we want, usually on the
 *  default page view.
 *  @param $embed
 *    The video code for the video to embed.
 *  @param $width
 *    The width to display the video.
 *  @param $height
 *    The height to display the video.
 *  @param $field
 *    The field info from the requesting node.
 *  @param $item
 *    The actual content from the field.
 *  @return
 *    The html of the embedded video.
 */
function emvideo_universalsubtitles_video($embed, $width, $height, $field, $item, $node, $autoplay) {
  $output = theme('emvideo_universalsubtitles_flash', $item, $width, $height, $autoplay);
  return $output;
}

/**
 *  hook emvideo_PROVIDER_video
 *
 *  This actually displays the preview-sized video we want, commonly for the
 *  teaser.
 *  @param $embed
 *    The video code for the video to embed.
 *  @param $width
 *    The width to display the video.
 *  @param $height
 *    The height to display the video.
 *  @param $field
 *    The field info from the requesting node.
 *  @param $item
 *    The actual content from the field.
 *  @return
 *    The html of the embedded video.
 */
function emvideo_universalsubtitles_preview($embed, $width, $height, $field, $item, $node, $autoplay) {
  $output = theme('emvideo_universalsubtitles_flash', $item, $width, $height, $autoplay);
  return $output;
}

/**
 *  Implementation of hook_emfield_subtheme.
 *  This returns any theme functions defined by this provider.
 */
function emvideo_universalsubtitles_emfield_subtheme() {
  $themes = array(
    'emvideo_universalsubtitles_flash'  => array(
       'arguments' => array('item' => NULL, 'width' => NULL, 'height' => NULL, 'autoplay' => NULL),
       'file' => 'providers/universalsubtitles.inc',
       'path' => drupal_get_path('module', 'media_universalsubtitles'),
     )
  );
  return $themes;
}
